!+
! Program:  epframes/test/checkoutput
!
! Authors:  Matthias Ehle, Michael Freyberg - XMM-SSC-MPE
!          
! Description: Tests output and functionality of epframes
! 
!- 

program checkoutput_no

use dal
use errorhandling

implicit none

type(DataSetT)  :: inevents, ingti
type(TableT)    :: eventstable, expotable, gtitable
type(ColumnT)   :: timeCol, rawXCol, rawYCol, flagCol, ccdnrCol, &
		   startCol, stopCol

real(kind=double),  dimension(:), pointer :: time, start, stop
integer(kind=Int8) ,dimension(:), pointer :: ccdnr
integer(kind=Int16),dimension(:), pointer :: rawX, rawY
integer(kind=Int32),dimension(:), pointer :: flag

real(kind=double)                 :: deltat, halfdt, maxerr
integer                           :: i, nevrows, ngtirows, CCDno
integer,parameter                 :: itest = 1  ! no
logical(kind=bool)                :: error_in_test, empty_frames
character(len=16),dimension(4)    :: list = (/"rawevents_no.dat", &
                                              "rawevents_IM.dat", &
                                              "rawevents_TI.dat", &
				              "rawevents_BU.dat"/)
character(len=6),dimension(4)     :: extlist = (/"EVENTS", &
                                                 "EVENTS", &
                                                 "EVENTS", &
				        	 "EVENTS"/)
character(len=10),dimension(4)    :: gtilist = (/"gti_no.dat", &
                                                 "gti_IM.dat", &
                                                 "gti_TI.dat", &
				        	 "gti_BU.dat"/)	
      
character(len=20)                 :: expname, gtiname, readoutmode

maxerr = 1.e-9 ! allowed discrepancy in time computation [1ns]
maxerr = 1.e-4 ! allowed discrepancy in time computation [0.1ms]

error_in_test = .false.
empty_frames  = .false.

  if (itest == 1 .or. itest == 2) readoutmode = 'IMAGING'
  if (itest == 3) readoutmode = 'TIMING'
  if (itest == 4) readoutmode = 'BURST'
  
  write (*,*) 'testing epframes ',trim(readoutmode),' mode output:'
  
  !-----------------------------------------
  write (*,*) 'testing ',list(itest),' ...'
  !-----------------------------------------

  inevents = dataSet(list(itest), read)

  !-----------------------------------

  eventstable = table(inevents,extlist(itest))

  if (.not.hasColumn(eventstable,"TIME")  .or. &
      .not.hasColumn(eventstable,"FRAME") .or. &
      .not.hasColumn(eventstable,"RAWX")  .or. &
      .not.hasColumn(eventstable,"RAWY")  .or. &
      .not.hasColumn(eventstable,"PHA")   .or. &
      .not.hasColumn(eventstable,"CCDNR") .or. &
      .not.hasColumn(eventstable,"FLAG")) &
      call error("noEvColumn", &
                 "Events extension: column absent or incorrectly named")

  write(expname,'(a)') "EXPOSURE"
  expotable = table(inevents,expname)

  if (.not.hasColumn(expotable,"TIME")    .or. &
!     .not.hasColumn(expotable,"TIMEDEL") .or. &
      .not.hasAttribute(expotable,"TIMEDEL") .or. &
      .not.hasColumn(expotable,"FRACEXP")) &
      call error("noExpoColumn", &
               "Exposure extension: column absent or incorrectly named")			

  !-----------------------------------
  
  if (itest /= 1) then

    rawXCol  = column(eventstable, "RAWX" ,read)
    rawYCol  = column(eventstable, "RAWY" ,read)
    flagCol  = column(eventstable, "FLAG" ,read)

    nevrows = numberofRows(eventstable)

    rawX => int16Data(rawXCol)
    rawY => int16Data(rawYCol)
    flag => int32Data(flagCol)

    do i = 0,  nevrows-1
      if (rawX(i) < 1 .or. rawY(i) < 1 .or. &
          rawX(i) > 64 .or. rawY(i) > 200) then
        if (flag(i) == 0) then
           write(errstr,*) "Coordinates out of range: rawX/Y(",i+1,")=(",&
                    rawX(i),",",rawY(i),") but not flagged: flag(",i,")=",flag(i)
           error_in_test = .true.
           call warning("wrongCoordsFlag")
        endif
      endif
    enddo

    if (hasAttribute(eventstable,"EMPTYFRM")) then
      empty_frames = (trim(stringAttribute(eventstable,"EMPTYFRM")) == 'enabled')
    endif

    !---------------------------------------------
    write (*,*) 'testing ',gtilist(itest),' ...'
    !---------------------------------------------

    ingti = dataSet(gtilist(itest),read)

    !-----------------------------------

    ccdnrCol =  column(eventstable, "CCDNR" ,read)
    ccdnr    => int8Data(ccdnrCol)
    CCDno    =  ccdnr(0)

    write(gtiname,'(a,i2.2)') "STDGTI",CCDno

    gtitable = table(ingti,gtiname)

    if (.not.hasColumn(gtitable,"START") .or. &
        .not.hasColumn(gtitable,"STOP")) &
        call error("noGTIColumn", &
                            "GTI file: column absent or incorrectly named")

    !-----------------------------------

    timeCol  = column(eventstable,"TIME",read)
    startCol = column(gtitable,  "START",read)
    stopCol  = column(gtitable,   "STOP",read)

    ngtirows = numberofRows(gtitable)

    if (ngtirows == 0) then 
      call warning("noGTIs","There are no GTI rows")
    else
      time  => real64Data(timeCol)
      start => real64Data(startCol)
      stop  => real64Data(stopCol)

      deltat = real64Attribute(gtitable,"FRMTIME")/1000.
      halfdt = deltat/2

      do i = 0, nevrows-1
         if (flag(i) == 0) then
!           if (abs(start(0) - (time(i) - halfdt)) > maxerr) &
            if (abs(start(0) - (time(i) - 2*halfdt)) > maxerr) then
              write(errstr,*) 'i, start(0), time(i), diff = ', &
                   i, start(0), time(i), &
                   abs(start(0) - (time(i) - 2*halfdt))
              call message(AppMsg,Sparse)
              call warning ("wrongGTI","Wrong GTI start calculation")
              if (.not. empty_frames) error_in_test = .true.
              exit
            endif
         endif
      enddo

      do i = nevrows-1, 0, -1
        if (flag(i) == 0) then
!           if (abs(stop(ngtirows-1) - (time(i) + halfdt)) > maxerr) &
            if (abs(stop(ngtirows-1) - (time(i) + 0*halfdt)) > maxerr) then
              write(errstr,*) 'i, stop(ngtirows-1), time(i), diff = ', &
                   i,stop(ngtirows-1) , time(i), &
                   abs(stop(ngtirows-1) - (time(i) + 0*halfdt))
              call message(AppMsg,Sparse)
              call warning ("wrongGTI","Wrong GTI stop calculation")
              if (.not. empty_frames) error_in_test = .true.
              exit
            endif
         endif
      enddo

    endif

    call release(ingti)

  endif  

  call release(inevents)

  if (error_in_test) then
    write(errstr,*) 'Test not successful for ITEST = ',itest
    call error("failure")
  else
    call message(AppMsg,Verbose,' ... testing successful')
  endif

end
